<?php

//  Copyright (C) 2002-2006 Balint Kis (balint@k-i-s.net)
//  Copyright (C) 2005-2022 Solomon Peachy (pizza@shaftnet.org)

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

function convert_xml2html($string) {
  $string = ereg_replace("<", "&lt;", $string);
  $string = ereg_replace(">", "&gt;", $string);
  $string = ereg_replace("\"", "&quot;", $string);
  /* replace \n with <br/>*/
  $string = ereg_replace("\n", "<br/>", $string);
  return $string;
}

function create_xml_header() {
  $xml_string = "<?xml version='1.0'?>\n";
  $xml_string .= "<!DOCTYPE BULKUPLOAD SYSTEM \"http://po.shaftnet.org/_media/po.dtd/\" \"2.36\">\n";
  return $xml_string;
}

function create_xml_bulkupload_header() {
  return "<bulkupload>\n";
}

function create_xml_folder_header($folder_id, $folder_name) {
  $folder_name = htmlspecialchars($folder_name, ENT_XML1, 'UTF-8');
  return "<folder id=\"$folder_id\" name=\"$folder_name\">\n";
}

function create_xml_folder_footer() {
  return "</folder>\n";
}

function create_xml_bulkupload_footer() {
  return "</bulkupload>\n";
}

function create_xml_photo($image_data) {
  $xml_string = "<photo>\n";

  foreach($image_data['file'] as $version) {
    $xml_string .= "<file";
    if ($version['master'] == 't')
      $xml_string .= " master=\"yes\"";
    else
      $xml_string .= " master=\"no\"";

    $xml_string .= " name=\"".$version['name']."\"";

    if ($version['remark'])
      $xml_string .= " remark=\"".$version['remark']."\"";

    $xml_string .= " />\n";
  }

  if ($image_data['author'])
    $xml_string .= "<author>".$image_data['author']."</author>\n";
  if (count($image_data['keywords'])) {
    foreach ($image_data['keywords'] as $keyword) {
      $xml_string .= "<keyword>".$keyword."</keyword>\n";
    }
  }
  if ($image_data['caption'])
    $xml_string .= "<caption>".$image_data['caption']."</caption>\n";
  if ($image_data['copyright'])
    $xml_string .= "<copyright>".$image_data['copyright']."</copyright>\n";
  if ($image_data['web_statement'])
    $xml_string .= "<web_statement>".$image_data['web_statement']."</web_statement>\n";
  if ($image_data['caption_writer'])
    $xml_string .= "<caption_writer>".$image_data['caption_writer']."</caption_writer>\n";
  if ($image_data['supplemental_category'])
    $xml_string .= "<supplemental_category>".$image_data['supplemental_category']."</supplemental_category>\n";
  if ($image_data['credit'])
    $xml_string .= "<credit>".$image_data['credit']."</credit>\n";
  if ($image_data['source'])
    $xml_string .= "<source>".$image_data['source']."</source>\n";
  if ($image_data['headline'])
    $xml_string .= "<headline>".$image_data['headline']."</headline>\n";
  if ($image_data['instructions'])
    $xml_string .= "<instructions>".$image_data['instructions']."</instructions>\n";
  if ($image_data['transmission_reference'])
    $xml_string .= "<transmission_reference>".$image_data['transmission_reference']."</transmission_reference>\n";
  if ($image_data['remark'])
    $xml_string .= "<remark>".$image_data['remark']."</remark>\n";
  if ($image_data['title'])
    $xml_string .= "<title>".$image_data['title']."</title>\n";
  if ($image_data['store_url'] != "") {
    $xml_string .= "<purchase_url>$image_data[store_url]</purchase_url>\n";
  }

  if ($image_data['date_of_exposure'] != "null") {
    $exposure_year  = substr($image_data['date_of_exposure'], 0, 4);
    $exposure_month = substr($image_data['date_of_exposure'], 5, 2);
    $exposure_day   = substr($image_data['date_of_exposure'], 8, 2);
    $exposure_hour  = substr($image_data['date_of_exposure'], 11, 2);
    $exposure_min   = substr($image_data['date_of_exposure'], 14, 2);
    $exposure_sec   = substr($image_data['date_of_exposure'], 17, 2);
    if ($exposure_year || $exposure_month || $exposure_day) {
      $xml_string .= "<expdate sqldate=\"$image_data[date_of_exposure]\" ";
      if ($exposure_year)
	$xml_string .= " year=\"$exposure_year\"";
      if ($exposure_month)
	$xml_string .= " month=\"$exposure_month\"";
      if ($exposure_day)
	$xml_string .= " day=\"$exposure_day\"";
      if ($exposure_hour)
	$xml_string .= " hours=\"$exposure_hour\"";
      if ($exposure_min)
	$xml_string .= " minutes=\"$exposure_min\"";
      if ($exposure_sec)
      $xml_string .= " seconds=\"$exposure_sec\"";
      $xml_string .= " />\n";
    }
  }

  if ($image_data['location'] && $image_data['location'] != "null")
    $xml_string .= "<location id=\"".$image_data['location']."\" />\n";

  if ($image_data['latitude'] || $image_data['longitude'] || 
      $image_data['altitude'] || $image_data['direction']) {
    $xml_string .= "<geolocation ";

    if ($image_data['latitude']) $xml_string .= "latitude=\"$image_data[latitude]\" ";
    if ($image_data['altitude']) $xml_string .= "altitude=\"$image_data[altitude]\" ";
    if ($image_data['longitude']) $xml_string .= "longitude=\"$image_data[longitude]\" ";
    if ($image_data['direction']) $xml_string .= "direction=\"$image_data[direction]\" ";
    
    $xml_string .= "/>\n";
  }

  if (($image_data['access_rights'] && $image_data['access_rights'] != "null") ||
      ($image_data['hide_original'] && $image_data['hide_original'] != "null")) {
    $xml_string .= "<access";
    if (($image_data['access_rights'] && $image_data['access_rights'] != "null"))
      $xml_string .= " type=\"".$image_data['access_rights']."\"";
    if (($image_data['hide_original'] && $image_data['hide_original'] != "null"))
      $xml_string .= " original=\"".$image_data['hide_original']."\"";
    $xml_string .= " />\n";
  }


  if (($image_data['camera'] && $image_data['camera'] != "null") ||
      ($image_data['camera_metering'] && $image_data['camera_metering'] != "null") ||
      ($image_data['camera_program'] && $image_data['camera_program'] != "null") ||
      ($image_data['shutter'] && $image_data['shutter'] != "null") ||
      ($image_data['exp_comp'] && $image_data['exp_comp'] != "null" && $image_data['exp_comp'] != "0.0") ||
      ($image_data['exp_diff'] && $image_data['exp_diff'] != "null" && $image_data['exp_diff'] != "0.0")) {
    $xml_string .= "<camera";
    if ($image_data['camera'] && $image_data['camera'] != "null")
      $xml_string .= " id=\"".$image_data['camera']."\"";
    if ($image_data['camera_metering'] && $image_data['camera_metering'] != "null")
      $xml_string .= " meter=\"".$image_data['camera_metering']."\"";
    if ($image_data['camera_program'] && $image_data['camera_program'] != "null")
      $xml_string .= " prog=\"".$image_data['camera_program']."\"";
    if ($image_data['shutter'] && $image_data['shutter'] != "null")
      $xml_string .= " speed=\"".$image_data['shutter']."\"";
    if ($image_data['exp_comp'] && $image_data['exp_comp'] != "null" && $image_data['exp_comp'] != "0.0")
      $xml_string .= " comp=\"".$image_data['exp_comp']."\"";
    if ($image_data['exp_diff'] && $image_data['exp_diff'] != "null" && $image_data['exp_diff'] != "0.0")
      $xml_string .= " evdiff=\"".$image_data['exp_diff']."\"";
    $xml_string .= " />\n";
  }

  if (($image_data['film'] && $image_data['film'] != "null") ||
      ($image_data['iso_override'] && $image_data['iso_override'] != "null")) {
    $xml_string .= "<film";
    if ($image_data['film'] && $image_data['film'] != "null")
      $xml_string .= " id=\"".$image_data['film']."\"";
    if ($image_data['iso_override'] && $image_data['iso_override'] != "null")
      $xml_string .= " iso_override=\"".$image_data['iso_override']."\"";
    $xml_string .= " />\n";
  }

  foreach ($image_data['equipment'] as $item) {
    $parts = explode(":", $item);

    if ($parts[1] == PO_EQUIPMENT_FILTER) {
      $xml_string .= "<filter id=\"$parts[0]\" />\n";
    }

    if ($parts[1] == PO_EQUIPMENT_SUPPORT) {
      $xml_string .= "<support id=\"$parts[0]\" />\n";
    }

    if ($parts[1] == PO_EQUIPMENT_LENS) {
      $xml_string .= "<lens id=\"$parts[0]\" ";

      if ($image_data['focal_length'] && $image_data['focal_length'] != "null")
	$xml_string .= " focal_length=\"".$image_data['focal_length']."\"";
      if ($image_data['aperture'] && $image_data['aperture'] != "null")
	$xml_string .= " aperture=\"".$image_data['aperture']."\"";

      $xml_string .= " />\n";
    }

    if ($parts[1] == PO_EQUIPMENT_FLASH) {
      $xml_string .= "<flash id=\"$parts[0]\" ";
      if ($image_data['flash_mode'] && $image_data['flash_mode'] != "null")
	$xml_string .= " mode=\"".$image_data['flash_mode']."\"";
      if ($image_data['flash_comp'] && $image_data['flash_comp'] != "null" && $image_data['flash_comp'] != "0.0")
	$xml_string .= " comp=\"".$image_data['flash_comp']."\"";
      $xml_string .= " />\n";
    }

    if ($parts[1] == PO_EQUIPMENT_SCANNER) {
      $xml_string .= "<scanner id=\"$parts[0]\" ";
      if ($image_data['scan_resolution'] && $image_data['scan_resolution'] != "null")
	$xml_string .= " dpi=\"".$image_data['scan_resolution']."\"";
      if ($image_data['scan_bitdepth'] && $image_data['scan_bitdepth'] != "null")
	$xml_string .= " bitdepth=\"".$image_data['scan_bitdepth']."\"";
      if ($image_data['scan_multiscan'] && $image_data['scan_multiscan'] != "null")
	$xml_string .= " passes=\"".$image_data['scan_multiscan']."\"";
      $xml_string .= " />\n";
    }
  }

  $xml_string .= "</photo>\n";

  return $xml_string;
}

?>
