#!/usr/bin/perl
###############
#
# (c) 2004-2007 Solomon Peachy.  See GPLv3 for license terms.
#
# Phase 2 of the backup system -- generate a file list for a backup 
# volume and creates an ISO image of that file list ready to be burned 
# onto a CD/DVD.  By default we assume DVD-DL disk, but you can use any 
# size you want.  See the '$max' variable below.
###############

use strict;
use DBI;

my ($size, $max, $count, $block);

# configuration

my $root = "/mnt/bulk/peachyphotos_repo";   # points to PO repository
my $db_backup = "db.backup.xz";  # Lives in PO repo
my $db_dsn = "dbname=photo"; # db dsn
my $db_pass = "apache"; # db pass
my $db_user = "apache"; # db user
my $dest_dir = "/home/pizza"; # where we shove the ISO

$block = 2048;  # CD/DVD/BD uses 2K blocks
#$max =  700*1024*1024; # 700 MiB, (CD)
#$max = 4700*1000*1000; # 4700 MB, (DVD)
#$max = 8500*1000*1000; # 8500 MB, (DVD-DL)
$max = 25000*1000*1000 - 256*1024*1024; # 25GB, (Blu-Ray)
#$max = 50000*1000*1000 - 256*1024*1024; # 50GB, (Blu-Ray DL)

$max -= 10*1024*1024;   # Leave 10MiB free to account for block slop.

my $where = "size = 0"; # only include originals, not scaled versions

my ($dbh, $select, $stmt, $stmt2);
my ($index, $indexfile, $flistfile, @flist, $volume);

my $temp;

$index = $ARGV[0];
$indexfile = sprintf ("index-photos-%04d", $index);
$flistfile = sprintf ("flist-photos-%04d", $index);
$volume = sprintf ("photos-%04d", $index);
$count = 0;
$size = 0;

if ($where) {
	$where .= " AND ";
}
$where .= " backup_volume IS NULL ";

# Account for DB backup size
$size += ((stat("$root/$db_backup"))[7] + $block - 1) / $block * $block;

# hook up to database
$dbh = DBI->connect("dbi:Pg:$db_dsn",
		    $db_user,
		    $db_pass,
		    { AutoCommit => 0}) || die($DBI::errstr);
$dbh->{RaiseError} = 1;

# load it all up from the database!
$stmt = $dbh->prepare("SELECT identifier, path, filesize, sha1sum FROM files WHERE $where ORDER by path");
$stmt2 = $dbh->prepare("UPDATE files SET backup_volume = ? WHERE identifier = ?");

open (INDEX_FILE, ">$indexfile") || die ("can't open $indexfile for writing");

$stmt->execute();
while (my $href = $stmt->fetchrow_hashref()) {
    my $filesize = $$href{"filesize"};
    $filesize = (($filesize+$block-1)/$block)*$block;    
    if (($size+$filesize) < $max) {
	if (! -f "$root/" . $$href{"path"} ) {
		print STDERR "MISSING: '" . $$href{"path"} . "'\n";
		next;
	}
	print INDEX_FILE $$href{"sha1sum"} . "  " . $$href{"path"} . "\n";
	push (@flist, $$href{"path"});
	$size += $filesize;
	$count++;
	$stmt2->execute($volume, $$href{"identifier"});
    } else {
	last;
    }
}
$stmt->finish();

print INDEX_FILE "# $indexfile -- count: $count total bytes: $size\n";
close(INDEX_FILE);

$stmt2->finish();

$dbh->commit();
$dbh->disconnect();

open (FLIST_FILE, ">$flistfile") || die ("can't open $flistfile for writing");

foreach (@flist) {
    /(.*)\/(.*)/;
    print FLIST_FILE "$1/=$root/$_\n";
}

print FLIST_FILE "/=$root/$db_backup\n";
print FLIST_FILE "$indexfile\n";
close (FLIST_FILE);

my $cmd = "growisofs -dvd-compat -Z /dev/sr0 -r -v -J -V $volume -graft-points -path-list $flistfile";
print "==> $cmd <==\n";

system($cmd);
